#!/bin/bash

readonly VERSION_FILE="$(dirname $0)/VERSION"
readonly CONFIG_FILE="$(dirname $0)/CONFIG"
readonly DL_FILE=("NEW_VERSION" "uImage-x1-" "armadillo_")

Syslog_write() {
    logger -t "auto-updater[$$]" -p user.notice ${1}
}

# version file existence check
Existence_check() {
    if [[ ! -f ${1} ]]; then
        Syslog_write "${1} file not found."
        exit 1
    fi
}

# download image file
File_download() {
    wget -q http://${DL_FILE_IP}/${1}
    if [[ $? -gt 0 ]]; then
        Syslog_write "Download ${1} failed."
        exit 1
    fi
}

# Get data of Config and Version File
Get_Data() {
    export $(cat ${1} | grep -v ^\# | xargs)
}

# Check whether there is already a download file.
File_del() {
    # Download File Delete
    for file_data in ${DL_FILE[@]}
    do
        dir_count=`find . -type f -name "${file_data}*" | wc -l`
        if [[ $dir_count -gt 0 ]]; then
            rm ${file_data}*
        fi
    done
}

# Check if the file to be mounted is already mounted.
Mount_check() {
    # mount check
    if [ `mount | grep ${MOUNT_DIR}` ]; then
        umount ${MOUNT_DIR}
    fi
}

# download version check
Version_check() {
    # same check
    if [ ${VERSION} = ${NEW_VERSION} ]; then
        Syslog_write "version the same."
        exit 0
    fi

    # 4.9 or 3.14 check
    if [ `echo ${VERSION} | grep v4.9-at` ]; then
        if [ -z `echo ${NEW_VERSION} | grep v4.9-at` ]; then
            Syslog_write "Linux 4.9 new version is wrong."
            exit 1
        fi
    elif [ `echo ${VERSION} | grep ^v` ]; then
        if [ -z `echo ${NEW_VERSION} | grep ^v` ]; then
            Syslog_write "Linux 3.14 new version is wrong."
            exit 1
        fi
    else
        Syslog_write "old version is wrong."
        exit 1
    fi
}

# Armadillo Update
Update_process() {
    uImage_name=${1}
    dtb_name=${2}

    mount -t vfat /dev/mmcblk2p1 ${MOUNT_DIR}
    cp ${uImage_name} ${MOUNT_DIR}/uImage
    dtb_basename=`basename $(ls ${MOUNT_DIR}/armadillo_*)`
    cp ${dtb_name} ${MOUNT_DIR}/${dtb_basename}
    umount ${MOUNT_DIR}
}

# main

# file init
File_del

# config file
Existence_check ${CONFIG_FILE}
Get_Data ${CONFIG_FILE}
if [[ -z ${DL_FILE_IP} || -z ${TARGET} || -z ${MOUNT_DIR} ]]; then
    Syslog_write "config value is not set."
    exit 1
fi

# version file
Existence_check ${VERSION_FILE}
Get_Data ${VERSION_FILE}
if [ -z ${VERSION} ]; then
    Syslog_write "version value is not set."
    exit 1
fi

# new version file
File_download ${DL_FILE[0]}
Existence_check ${DL_FILE[0]}
Get_Data ${DL_FILE[0]}
if [ -z ${NEW_VERSION} ]; then
    Syslog_write "new version value is not set."
    exit 1
fi

# mount check
Mount_check

# version check
Version_check

# Download Start.
# uImage download
File_download "${DL_FILE[1]}${NEW_VERSION}"
File_download "${DL_FILE[2]}${TARGET}-${NEW_VERSION}.dtb"

# Update
Update_process "${DL_FILE[1]}${NEW_VERSION}" "${DL_FILE[2]}${TARGET}-${NEW_VERSION}.dtb"

# write new Version
echo "VERSION=${NEW_VERSION}" > ${VERSION_FILE}

Syslog_write "Update completed."

sync
/sbin/reboot